<?php

namespace Drupal\ocr_image;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\ocr_image\Plugin\Field\TesseractLanguages;
use Symfony\Polyfill\Intl\Normalizer\Normalizer;
use thiagoalessio\TesseractOCR\TesseractOCR;

/**
 * OCR image service.
 */
class OcrImage {

  use StringTranslationTrait;

  /**
   * Logger Factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructor for OcrImage service.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   Logger.
   */
  public function __construct(LoggerChannelFactoryInterface $loggerFactory) {
    $this->loggerFactory = $loggerFactory;
  }

  /**
   * Returns string from image.
   *
   * @param string $file_path
   *   The path for the image to extract text from.
   * @param string $lang
   *   Language to ocr.
   * @param int $limit
   *   Limited characters.
   * @param int $dpi
   *   Dpi.
   *
   * @return array
   *   The array string line text.
   */
  public function getText(string $file_path, string $lang = 'eng', int $limit = 0, int $dpi = 300) {
    $ocr = new TesseractOCR($file_path);
    $data = '';
    $text = [];

    if (empty($lang)) {
      $lang = TesseractLanguages::tesseractConvertLang();
    }

    try {
      $data = $ocr->lang($lang)->dpi($dpi)->run();
      if ($limit > 0) {
        $data = substr($data, 0, $limit);
      }
    }
    catch (\Exception $e) {
      $error = $this->t('TesseractOCR not found');
      $this->loggerFactory('ocr_image')->error($error);
    }

    $data = Normalizer::normalize(trim($data), Normalizer::FORM_C);
    $line_datas = array_filter(explode("\n", $data), function ($value) {
      return !empty(trim($value));
    });

    if (!empty($line_datas)) {
      // Set the full text.
      $text['full_text'] = $data;

      // Set the line datas.
      $text['array'] = $line_datas;

      // Set the title.
      $title = trim(strip_tags(array_shift($line_datas)));
      $text['title'] = substr($title, 0, 512);

      // Set the alt.
      $alt = trim(strip_tags(implode(' ', $line_datas)));
      $text['alt'] = substr($alt, 0, 1024);
    }
    return $text;
  }

}
